/*
 * File: MyGame.js 
 * This is the logic of our game. For now, this is very simple.
 */
/*jslint node: true, vars: true */
/*global gEngine, SimpleShader, SquareRenderable, SceneNode, ArmSegment */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";  // Operate in Strict mode such that variables must be declared before used!

function ClassExample() {
    this.mSizeChange = 0.01;

    this.mConstColorShader = new SimpleShader(
        "src/GLSLShaders/SimpleVS.glsl",      // Path to the VertexShader 
        "src/GLSLShaders/SimpleFS.glsl");    // Path to the simple FragmentShader

    this.mParent = new SceneNode(this.mConstColorShader, "Root", true);
    this.mLeftChild = new ArmSegment(this.mConstColorShader, "LeftGen 1",
                            -2, 0);
    this.mParent.addAsChild(this.mLeftChild);
    this.mTopChild = new ArmSegment(this.mConstColorShader, "LeftGen 2",
                            -2, 2);
    this.mLeftChild.addAsChild(this.mTopChild);

    this.mRightChild = new ArmSegment(this.mConstColorShader, "RightGen 1",
                            2, 0);
    this.mParent.addAsChild(this.mRightChild);  // <-- WHAT ARE WE DOING?!!


    // shapes in the parent
    var obj = new SquareRenderable(this.mConstColorShader);  // the base
    this.mParent.addToSet(obj);
    obj.setColor([0.3, 0.3, 0.9, 1]);
    var xf = obj.getXform();
    xf.setSize(4, 1.5);
    
    obj = new SquareRenderable(this.mConstColorShader); // The head
    this.mParent.addToSet(obj);
    obj.setColor([0.9, 0.8, 0.8, 1]);
    xf = obj.getXform();
    xf.setSize(1.3, 1.3);
}

ClassExample.prototype.draw = function (camera) {
    // Step F: Starts the drawing by activating the camera
    camera.setupViewProjection();

    this.mParent.draw(camera);
};

ClassExample.prototype.update = function () {
    this.mLeftChild.update();
    this.mRightChild.update();
    this.mTopChild.update();
    
    // 1. rotate the head (middle square on body)
    var xf = this.mParent.getRenderableAt(1).getXform(); // this is the middle square
    xf.incRotationByDegree(2);
    var d = xf.getRotationInDegree();
    
    // 2. extend the lower-left hand
    xf = this.leftChildXform();
    xf.incSizeBy(this.mSizeChange);
    if ((xf.getWidth() > 1.2) || (xf.getWidth() < 0.8))
        this.mSizeChange = -this.mSizeChange;
    
    // 3. slowly rotat the top arm
    xf = this.topChildXform();
    xf.incRotationByDegree(-1);
};



ClassExample.prototype.leftChildXform = function () {
    return this.mLeftChild.getXform();
};

ClassExample.prototype.rightChildXform = function () {
    return this.mRightChild.getXform();
};


ClassExample.prototype.topChildXform = function () {
    return this.mTopChild.getXform();
};


ClassExample.prototype.parentXform = function () {
    return this.mParent.getXform();
};